# translation.py

import argparse
from transformers import MarianMTModel, MarianTokenizer, M2M100ForConditionalGeneration, M2M100Tokenizer


def _parse_args():
    """
    Command-line arguments to the system. --model switches between the main modes you'll need to use. The other arguments
    are provided for convenience.
    :return: the parsed args bundle
    """
    parser = argparse.ArgumentParser(description='translation.py')
    parser.add_argument('--input_sent', type=str, default="", help="sentence to translate")
    parser.add_argument('--use_m2m', default=False, action='store_true',
                        help="Use M2M instead of Marian")
    parser.add_argument('--marian_model', type=str, default="Helsinki-NLP/opus-mt-fr-en", help='path to model checkpoint')
    args = parser.parse_args()
    return args


def use_m2m(src_text, tokenizer, model, src_lang, trg_lang):
    """
    Prints results from beam search and sampling from an M2M100 model
    https://huggingface.co/docs/transformers/model_doc/m2m_100
    :param src_text: The source text (single sentence) to translate
    :param tokenizer: The M2M100 tokenizer
    :param model: The M2M100 model
    :param src_lang: source language code (fr for French)
    :param trg_lang: target language code (en for English)
    :return:
    """
    tokenizer.src_lang = src_lang
    encoded_text = tokenizer(src_text, return_tensors="pt")
    generated_tokens = model.generate(**encoded_text, forced_bos_token_id=tokenizer.get_lang_id(trg_lang))
    res = tokenizer.batch_decode(generated_tokens, skip_special_tokens=True)
    print("Beam Size 1 (Greedy): " + repr(res))
    generated_tokens = model.generate(**encoded_text, num_beams=5, forced_bos_token_id=tokenizer.get_lang_id(trg_lang))
    res = tokenizer.batch_decode(generated_tokens, skip_special_tokens=True)
    print("Beam Size 5: " + repr(res))
    for i in range(0, 5):
        generated_tokens = model.generate(**encoded_text, do_sample=True, forced_bos_token_id=tokenizer.get_lang_id(trg_lang))
        res = tokenizer.batch_decode(generated_tokens, skip_special_tokens=True)
        print("Sampling: " + repr(res))
    for i in range(0, 5):
        generated_tokens = model.generate(**encoded_text, do_sample=True, top_p = 0.9, forced_bos_token_id=tokenizer.get_lang_id(trg_lang))
        res = tokenizer.batch_decode(generated_tokens, skip_special_tokens=True)
        print("Nucleus sampling: " + repr(res))


def use_marian(src_text, tokenizer, model):
    """
    Prints results from beam search and sampling from a MarianMT model
    https://huggingface.co/docs/transformers/model_doc/marian
    :param src_text: The text to translate (a single sentence)
    :param tokenizer: MarianTokenizer
    :param model: MarianMTModel
    :return:
    """
    src_text_list = [
        src_text
    ]
    encoded_text = tokenizer(src_text_list, padding=True)
    print("Input to the model: "+ repr(tokenizer.convert_ids_to_tokens(encoded_text['input_ids'][0])))

    encoded_text_pt = tokenizer(src_text_list, return_tensors="pt", padding=True)
    translated = model.generate(**encoded_text_pt)
    print("Beam Size 1 (Greedy): " + repr([tokenizer.decode(t, skip_special_tokens=True) for t in translated]))
    translated = model.generate(**encoded_text_pt, num_beams=5)
    print("Beam Size 5: " + repr([tokenizer.decode(t, skip_special_tokens=True) for t in translated]))
    for i in range(0, 5):
        translated = model.generate(**encoded_text_pt, do_sample=True)
        print("Sampling: " + repr([tokenizer.decode(t, skip_special_tokens=True) for t in translated]))
    for i in range(0, 5):
        translated = model.generate(**encoded_text_pt, do_sample=True, top_p = 0.9)
        print("Nucleus sampling: " + repr([tokenizer.decode(t, skip_special_tokens=True) for t in translated]))


if __name__=="__main__":
    args = _parse_args()
    print(args)
    if args.use_m2m:
        model = M2M100ForConditionalGeneration.from_pretrained("facebook/m2m100_418M")  # N.B. 1.94 G download
        tokenizer = M2M100Tokenizer.from_pretrained("facebook/m2m100_418M")
        use_m2m(args.input_sent, tokenizer, model, "fr", "en")
    else:
        tokenizer = MarianTokenizer.from_pretrained(args.marian_model)
        model = MarianMTModel.from_pretrained(args.marian_model)
        use_marian(args.input_sent, tokenizer, model)

